               
/*
#include <stdio.h>
#include <Arduino.h>
#include <EEPROM.h>
#include <SPI.h>
#include <avr/pgmspace.h>
*/



/* Sample application to demonstrat FT800 primitives, widgets and customized screen shots */

#include "FT_Platform.h"
#include "FT_App_RotaryDial.h"
//#include "FT_Font_Table.h"

#ifdef DISPLAY_RESOLUTION_QVGA
#define FT_DISP_WIDTH (320)
#define FT_DISP_HEIGHT (240)
#endif

#ifdef DISPLAY_RESOLUTION_WQVGA
#define FT_DISP_WIDTH (480)
#define FT_DISP_HEIGHT (272)
#endif
#ifdef DISPLAY_RESOLUTION_HVGA_PORTRAIT
#define FT_DISP_WIDTH (320)
#define FT_DISP_HEIGHT (480)
#endif
//#include<time.h>


#define STARTUP_ADDRESS	100*1024L
#define ICON_ADDRESS	250*1024L
//////////////////////////////////////////////////////


#ifdef FT900_PLATFORM
	FATFS FatFs;
#endif



#define SAMAPP_DELAY_BTW_APIS (1000)
#define SAMAPP_ENABLE_DELAY() Ft_Gpu_Hal_Sleep(SAMAPP_DELAY_BTW_APIS)
#define SAMAPP_ENABLE_DELAY_VALUE(x) Ft_Gpu_Hal_Sleep(x)
#define BLK_LEN	  1024

#define CLOCK_HANDLE	(0x01)

#define DIAL		1

ft_int32_t BaseTrackVal = 0,BaseTrackValInit = 0,BaseTrackValSign = 0,MemLoc=0;
/* Global variables for display resolution to support various display panels */
/* Default is WQVGA - 480x272 */
ft_int16_t FT_DispWidth = 480;
ft_int16_t FT_DispHeight = 272;
ft_int16_t FT_DispHCycle =  548;
ft_int16_t FT_DispHOffset = 43;
ft_int16_t FT_DispHSync0 = 0;
ft_int16_t FT_DispHSync1 = 41;
ft_int16_t FT_DispVCycle = 292;
ft_int16_t FT_DispVOffset = 12;
ft_int16_t FT_DispVSync0 = 0;
ft_int16_t FT_DispVSync1 = 10;
ft_uint8_t FT_DispPCLK = 5;
ft_char8_t FT_DispSwizzle = 0;
ft_char8_t FT_DispPCLKPol = 1;
ft_char8_t FT_DispCSpread = 1;
ft_char8_t FT_DispDither = 1;


const ft_uint8_t FT_DLCODE_BOOTUP[12] = 
{
    0,255,0,2,//GPU instruction CLEAR_COLOR_RGB
    7,0,0,38, //GPU instruction CLEAR
    0,0,0,0,  //GPU instruction DISPLAY
};


/* Global used for buffer optimization */
Ft_Gpu_Hal_Context_t host,*phost;




ft_uint32_t Ft_CmdBuffer_Index;
ft_uint32_t Ft_DlBuffer_Index;

#ifdef BUFFER_OPTIMIZATION
ft_uint8_t  Ft_DlBuffer[FT_DL_SIZE];
ft_uint8_t  Ft_CmdBuffer[FT_CMD_FIFO_SIZE];
#endif


PROGMEM char *info[] = {  "FT800 Rotary Dial Application",
                          "APP to demonstrate interactive rotary dial,",
                          "using Points, Track",
                          "& Stencil"
                       }; 
#define CALIBRATION_FONTSIZE 28

                       

ft_void_t Ft_App_WrCoCmd_Buffer(Ft_Gpu_Hal_Context_t *phost,ft_uint32_t cmd)
{
#ifdef  BUFFER_OPTIMIZATION
   /* Copy the command instruction into buffer */
   ft_uint32_t *pBuffcmd;
   pBuffcmd =(ft_uint32_t*)&Ft_CmdBuffer[Ft_CmdBuffer_Index];
   *pBuffcmd = cmd;
#endif
#ifdef ARDUINO_PLATFORM
   Ft_Gpu_Hal_WrCmd32(phost,cmd);
#endif
#ifdef FT900_PLATFORM
   Ft_Gpu_Hal_WrCmd32(phost,cmd);
#endif
   /* Increment the command index */
   Ft_CmdBuffer_Index += FT_CMD_SIZE;  
}

ft_void_t Ft_App_WrDlCmd_Buffer(Ft_Gpu_Hal_Context_t *phost,ft_uint32_t cmd)
{
#ifdef BUFFER_OPTIMIZATION  
   /* Copy the command instruction into buffer */
   ft_uint32_t *pBuffcmd;
   pBuffcmd =(ft_uint32_t*)&Ft_DlBuffer[Ft_DlBuffer_Index];
   *pBuffcmd = cmd;
#endif

#ifdef ARDUINO_PLATFORM
   Ft_Gpu_Hal_Wr32(phost,(RAM_DL+Ft_DlBuffer_Index),cmd);
#endif
#ifdef FT900_PLATFORM
   Ft_Gpu_Hal_Wr32(phost,(RAM_DL+Ft_DlBuffer_Index),cmd);
#endif
   /* Increment the command index */
   Ft_DlBuffer_Index += FT_CMD_SIZE;  
}


ft_void_t Ft_App_WrCoStr_Buffer(Ft_Gpu_Hal_Context_t *phost,const ft_char8_t *s)
{
#ifdef  BUFFER_OPTIMIZATION  
  ft_uint16_t length = 0;
  length = strlen(s) + 1;//last for the null termination
  
  strcpy(&Ft_CmdBuffer[Ft_CmdBuffer_Index],s);  

  /* increment the length and align it by 4 bytes */
  Ft_CmdBuffer_Index += ((length + 3) & ~3);  
#endif  
}

ft_void_t Ft_App_Flush_DL_Buffer(Ft_Gpu_Hal_Context_t *phost)
{
#ifdef  BUFFER_OPTIMIZATION    
   if (Ft_DlBuffer_Index > 0)
     Ft_Gpu_Hal_WrMem(phost,RAM_DL,Ft_DlBuffer,Ft_DlBuffer_Index);
#endif     
   Ft_DlBuffer_Index = 0;
   
}

ft_void_t Ft_App_Flush_Co_Buffer(Ft_Gpu_Hal_Context_t *phost)
{
#ifdef  BUFFER_OPTIMIZATION    
   if (Ft_CmdBuffer_Index > 0)
     Ft_Gpu_Hal_WrCmdBuf(phost,Ft_CmdBuffer,Ft_CmdBuffer_Index);
#endif     
   Ft_CmdBuffer_Index = 0;
}


FT_PROGMEM ft_prog_uint16_t sintab[] = {
0, 402, 804, 1206, 1607, 2009, 2410, 2811, 3211, 3611, 4011, 4409, 4807, 5205, 5601, 5997, 6392, 
6786, 7179, 7571, 7961, 8351, 8739, 9126, 9511, 9895, 10278, 10659, 11038, 11416, 11792, 12166, 12539,
12909, 13278, 13645, 14009, 14372, 14732, 15090, 15446, 15799, 16150, 16499, 16845, 17189, 17530, 17868,
18204, 18537, 18867, 19194, 19519, 19840, 20159, 20474, 20787, 21096, 21402, 21705, 22004, 22301, 22594, 
22883, 23169, 23452, 23731, 24006, 24278, 24546, 24811, 25072, 25329, 25582, 25831, 26077, 26318, 26556, 26789, 
27019, 27244, 27466, 27683, 27896, 28105, 28309, 28510, 28706, 28897, 29085, 29268, 29446, 29621, 29790, 29955, 
30116, 30272, 30424, 30571, 30713, 30851, 30984, 31113, 31236, 31356, 31470, 31580, 31684, 31785, 31880, 31970, 
32056, 32137, 32213, 32284, 32350, 32412, 32468, 32520, 32567, 32609, 32646, 32678, 32705, 32727, 32744, 32757, 
32764, 32767, 32764};

ft_int16_t SAMAPP_qsin(ft_uint16_t a)
{
  ft_uint8_t f;
  ft_int16_t s0,s1;

  if (a & 32768)
    return -SAMAPP_qsin(a & 32767);
  if (a & 16384)
      a = 32768 - a;
  f = a & 127;
  s0 = ft_pgm_read_word(sintab + (a >> 7));
  s1 = ft_pgm_read_word(sintab + (a >> 7) + 1);
  return (s0 + ((ft_int32_t)f * (s1 - s0) >> 7));
}

/* cos funtion */
ft_int16_t SAMAPP_qcos(ft_uint16_t a)
{
  return (SAMAPP_qsin(a + 16384));
}




static void polarxy(ft_uint32_t r, ft_uint16_t th, ft_uint16_t *x, ft_uint16_t *y,ft_uint16_t ox,ft_uint16_t oy)
{
	
		*x = (16 * ox + (((long)r * SAMAPP_qsin(th)) >> 11) + 16);
		*y = (16 * oy - (((long)r * SAMAPP_qcos(th)) >> 11));
}
ft_uint16_t da(ft_int32_t i)
{

	return (i+90) * 65536 / 360;
}


ft_uint32_t Load_RawDataFromfile(ft_uchar8_t ImageArrayname[],/* Image Array*/
								 ft_uint32_t ptr,/* Array Size*/
								 ft_uint32_t RamAddr,
								 ft_int16_t Handle,
								 ft_int16_t Format,
								 ft_int16_t Stride,
								 ft_int16_t Width,
								 ft_int16_t Height,
								 ft_uint8_t i
								 )
{
	
	ft_int32_t fsize=0;
	fsize = SAMAPP_Uncompressed_FileSize[i];

        RamAddr = (ft_uint32_t)(RamAddr + 7) & (~7);
	Ft_Gpu_Hal_WrCmd32(phost,  CMD_INFLATE);
	Ft_Gpu_Hal_WrCmd32(phost,  RamAddr);

#ifdef MSVC_PLATFORM 
        Ft_Gpu_Hal_WrCmdBuf(phost,ImageArrayname,ptr);
#endif
#ifdef ARDUINO_PLATFORM
	Ft_Gpu_Hal_WrCmdBufFromFlash(phost,ImageArrayname,ptr);
#endif

	Ft_App_WrCoCmd_Buffer(phost,BITMAP_HANDLE(Handle));
	Ft_App_WrCoCmd_Buffer(phost,BITMAP_SOURCE(RamAddr));
	Ft_App_WrCoCmd_Buffer(phost, BITMAP_LAYOUT(Format,Stride,Height));		
	Ft_App_WrCoCmd_Buffer(phost, BITMAP_SIZE(NEAREST,BORDER,BORDER,Width,Height));		
	Ft_App_WrCoCmd_Buffer(phost, VERTEX2II(0,0,Handle,0));				
		

	fsize = (fsize + 7) & (~7);//align the buffer to 8
	return (RamAddr + fsize);
}

static ft_uint8_t sk=0;

ft_uint8_t Read_Keys()
{
  static ft_uint8_t Read_tag=0,temp_tag=0,ret_tag=0;	
  Read_tag = Ft_Gpu_Hal_Rd8(phost,REG_TOUCH_TAG);
  ret_tag = NULL;
  if(Read_tag!=NULL)		
  {
    if(temp_tag!=Read_tag)
    {
      temp_tag = Read_tag;	
      sk = Read_tag;										
    }  
  }
  else
  {
    if(temp_tag!=0)
    {
      ret_tag = temp_tag;
    }  
    sk = 0;
  }
  return ret_tag;
}

ft_void_t Info()
{
  ft_uint16_t dloffset = 0,z;
// Touch Screen Calibration
  Ft_CmdBuffer_Index = 0;  
  Ft_Gpu_CoCmd_Dlstart(phost); 
  Ft_App_WrCoCmd_Buffer(phost,CLEAR(1,1,1));
  Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,255,255));
  Ft_Gpu_CoCmd_Text(phost,FT_DispWidth/2,FT_DispHeight/2,CALIBRATION_FONTSIZE,OPT_CENTERX|OPT_CENTERY,"Please tap on a dot");
  Ft_Gpu_CoCmd_Calibrate(phost,0);
  Ft_App_WrCoCmd_Buffer(phost,DISPLAY());
  Ft_Gpu_CoCmd_Swap(phost);
  Ft_App_Flush_Co_Buffer(phost);
  Ft_Gpu_Hal_WaitCmdfifo_empty(phost);
// Ftdi Logo animation  
  Ft_Gpu_CoCmd_Logo(phost);
  Ft_App_Flush_Co_Buffer(phost);
  Ft_Gpu_Hal_WaitCmdfifo_empty(phost);
  while(0!=Ft_Gpu_Hal_Rd16(phost,REG_CMD_READ)); 
//Copy the Displaylist from DL RAM to GRAM
  dloffset = Ft_Gpu_Hal_Rd16(phost,REG_CMD_DL);
  dloffset -=4; 
#ifdef FT_81X_ENABLE
  dloffset -= 2*4;//remove two more instructions in case of 81x
#endif
  
  Ft_Gpu_Hal_WrCmd32(phost,CMD_MEMCPY);
  Ft_Gpu_Hal_WrCmd32(phost,STARTUP_ADDRESS);
  Ft_Gpu_Hal_WrCmd32(phost,RAM_DL);
  Ft_Gpu_Hal_WrCmd32(phost,dloffset);
//Enter into Info Screen
  do
  {
    Ft_Gpu_CoCmd_Dlstart(phost);   
    Ft_Gpu_CoCmd_Append(phost,STARTUP_ADDRESS,dloffset);
//Reset the BITMAP properties used during Logo animation
    Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_A(256));
    Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_A(256));
    Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_B(0));
    Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_C(0));
    Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_D(0));
    Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_E(256));
    Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_F(0));  
//Display the information with transparent Logo using Edge Strip  
	
    Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(219,180,150));
    Ft_App_WrCoCmd_Buffer(phost,COLOR_A(220));
    Ft_App_WrCoCmd_Buffer(phost,BEGIN(EDGE_STRIP_A));
    Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(0,FT_DispHeight*16));
    Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(FT_DispWidth*16,FT_DispHeight*16));
    Ft_App_WrCoCmd_Buffer(phost,COLOR_A(255));
    Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(0,0,0));

    Ft_Gpu_CoCmd_Text(phost,FT_DispWidth/2,20,28,OPT_CENTERX|OPT_CENTERY,info[0]);
    Ft_Gpu_CoCmd_Text(phost,FT_DispWidth/2,60,26,OPT_CENTERX|OPT_CENTERY,info[1]);
    Ft_Gpu_CoCmd_Text(phost,FT_DispWidth/2,90,26,OPT_CENTERX|OPT_CENTERY,info[2]);  
    Ft_Gpu_CoCmd_Text(phost,FT_DispWidth/2,120,26,OPT_CENTERX|OPT_CENTERY,info[3]);  
    Ft_Gpu_CoCmd_Text(phost,FT_DispWidth/2,FT_DispHeight-30,26,OPT_CENTERX|OPT_CENTERY,"Click to play");

	
//	Check if the Play key and change the color
    if(sk!='P')
    Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,255,255));
    else
	{
		Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(100,100,100));		
	}
    Ft_App_WrCoCmd_Buffer(phost,BEGIN(FTPOINTS));   
	
    Ft_App_WrCoCmd_Buffer(phost,POINT_SIZE(20*16));
    Ft_App_WrCoCmd_Buffer(phost,TAG('P'));
    Ft_App_WrCoCmd_Buffer(phost,VERTEX2F((FT_DispWidth/2)*16,(FT_DispHeight-60)*16));
	 Ft_App_WrCoCmd_Buffer(phost,CLEAR(0,1,0));

	  Ft_App_WrCoCmd_Buffer(phost, COLOR_RGB(255, 0, 0) );
	Ft_App_WrCoCmd_Buffer(phost, STENCIL_OP(INCR,INCR) );
	Ft_App_WrCoCmd_Buffer(phost, COLOR_MASK(0,0,0,0) );//mask all the colors
	Ft_App_WrCoCmd_Buffer(phost, BEGIN(EDGE_STRIP_L));

	Ft_App_WrCoCmd_Buffer(phost, VERTEX2II(((FT_DispWidth/2)-14+10),(FT_DispHeight-77+5),0,0));
	Ft_App_WrCoCmd_Buffer(phost, VERTEX2II(((FT_DispWidth/2)-2+15),(FT_DispHeight-67+8),0,0));
	Ft_App_WrCoCmd_Buffer(phost, VERTEX2II(((FT_DispWidth/2)-14+10),(FT_DispHeight-57+11),0,0));
	Ft_App_WrCoCmd_Buffer(phost, VERTEX2II(((FT_DispWidth/2)-14+10),(FT_DispHeight-77+5),0,0));
	Ft_App_WrCoCmd_Buffer(phost, END());
	Ft_App_WrCoCmd_Buffer(phost, COLOR_MASK(1,1,1,1) );//enable all the colors
	Ft_App_WrCoCmd_Buffer(phost, STENCIL_FUNC(EQUAL,1,255));
	Ft_App_WrCoCmd_Buffer(phost, BEGIN(EDGE_STRIP_L));
	Ft_App_WrCoCmd_Buffer(phost, VERTEX2II(FT_DispWidth,0,0,0));
	Ft_App_WrCoCmd_Buffer(phost, VERTEX2II(FT_DispWidth,FT_DispHeight,0,0));
	//Ft_App_WrCoCmd_Buffer(phost, END());
	
	Ft_App_WrCoCmd_Buffer(phost, STENCIL_OP(KEEP,KEEP));
	Ft_App_WrCoCmd_Buffer(phost, STENCIL_FUNC(ALWAYS,0,255));
	Ft_App_WrCoCmd_Buffer(phost,LINE_WIDTH(1*16));
	//Ft_App_WrCoCmd_Buffer(phost, COLOR_RGB(0, 0, 0) );
	Ft_App_WrCoCmd_Buffer(phost, BEGIN(LINE_STRIP));

	Ft_App_WrCoCmd_Buffer(phost, VERTEX2II(((FT_DispWidth/2)-14+10),(FT_DispHeight-77+5),0,0));
	Ft_App_WrCoCmd_Buffer(phost, VERTEX2II(((FT_DispWidth/2)-2+15),(FT_DispHeight-67+8),0,0));
	Ft_App_WrCoCmd_Buffer(phost, VERTEX2II(((FT_DispWidth/2)-14+10),(FT_DispHeight-57+11),0,0));
	Ft_App_WrCoCmd_Buffer(phost, VERTEX2II(((FT_DispWidth/2)-14+10),(FT_DispHeight-77+5),0,0));
	//Ft_App_WrCoCmd_Buffer(phost, END());



    //Ft_App_WrCoCmd_Buffer(phost,VERTEX2II((FT_DispWidth/2)-14,(FT_DispHeight-75),14,4));
    Ft_App_WrCoCmd_Buffer(phost,DISPLAY());
    Ft_Gpu_CoCmd_Swap(phost);
    Ft_App_Flush_Co_Buffer(phost);
    Ft_Gpu_Hal_WaitCmdfifo_empty(phost);
  }while(Read_Keys()!='P');
/* wait until Play key is not pressed*/ 
  /* To cover up some distortion while demo is being loaded */
    Ft_App_WrCoCmd_Buffer(phost, CMD_DLSTART);
  Ft_App_WrCoCmd_Buffer(phost,CLEAR_COLOR_RGB(255,255,255));
  Ft_App_WrCoCmd_Buffer(phost,CLEAR(1,1,1));
  Ft_App_WrCoCmd_Buffer(phost,DISPLAY());
  Ft_Gpu_CoCmd_Swap(phost);
  Ft_App_Flush_Co_Buffer(phost);
  Ft_Gpu_Hal_WaitCmdfifo_empty(phost);
/* wait until Play key is not pressed*/ 
}


/* Definitions used for optimizations */
#define ROTARY_APP_WHITECIRCELE_FIFO_SIZE (4*6+4*7*10+4*1)
#define ROTARY_APP_START_FIFO_SIZE_INWORDS (59)
#define ROTARY_APP_START_FIFO_SIZE (4*ROTARY_APP_START_FIFO_SIZE_INWORDS)
#define ROTARY_APP_START_FIFO_RANDOMBITMAP_SIZE (4*4)
#define ROTARY_APP_END_FIFO_SIZE_INWORDS (48)
#define ROTARY_APP_END_FIFO_SIZE (4*ROTARY_APP_END_FIFO_SIZE_INWORDS)
#define ROTARY_APP_BLOCK1_SIZE_INWORDS (17)
#define ROTARY_APP_BLOCK1_SIZE (4*ROTARY_APP_BLOCK1_SIZE_INWORDS)

/* Array optimization for code space */
#if 1        
       FT_PROGMEM prog_uint32_t ROTARY_APP_DL_BLOCK0 [ROTARY_APP_START_FIFO_SIZE_INWORDS] =
        {	
          SCISSOR_SIZE(512,512),
	  SCISSOR_XY(0,0),
 	  /* Draw all the background bitmaps - balls with various resolutions */
	  BEGIN(BITMAPS),
	  COLOR_RGB(255 ,255 ,255), // brown outside circle
          VERTEX2II(0,0,0,0),VERTEX2II(0,0,0,0),VERTEX2II(0,0,0,0),VERTEX2II(0,0,0,0),VERTEX2II(0,0,0,0),
          VERTEX2II(0,0,0,0),VERTEX2II(0,0,0,0),VERTEX2II(0,0,0,0),VERTEX2II(0,0,0,0),VERTEX2II(0,0,0,0),
          VERTEX2II(0,0,0,0),VERTEX2II(0,0,0,0),VERTEX2II(0,0,0,0),VERTEX2II(0,0,0,0),VERTEX2II(0,0,0,0),
          VERTEX2II(0,0,0,0),VERTEX2II(0,0,0,0),VERTEX2II(0,0,0,0),VERTEX2II(0,0,0,0),VERTEX2II(0,0,0,0),
          VERTEX2II(0,0,0,0),VERTEX2II(0,0,0,0),VERTEX2II(0,0,0,0),VERTEX2II(0,0,0,0),VERTEX2II(0,0,0,0),
          VERTEX2II(0,0,0,0),VERTEX2II(0,0,0,0),VERTEX2II(0,0,0,0),VERTEX2II(0,0,0,0),VERTEX2II(0,0,0,0),
          VERTEX2II(0,0,0,0),VERTEX2II(0,0,0,0),VERTEX2II(0,0,0,0),VERTEX2II(0,0,0,0),VERTEX2II(0,0,0,0),
          VERTEX2II(0,0,0,0),VERTEX2II(0,0,0,0),VERTEX2II(0,0,0,0),VERTEX2II(0,0,0,0),VERTEX2II(0,0,0,0),
#ifdef DISPLAY_RESOLUTION_WQVGA
  	  POINT_SIZE(((FT_DISP_WIDTH/4) - 19) * 16),		
#endif
#ifdef DISPLAY_RESOLUTION_QVGA
  	  POINT_SIZE(((FT_DISP_WIDTH/4) + 5) * 16),		
#endif
	  BEGIN(FTPOINTS),
	  COLOR_RGB(235 ,117 ,0), // brown outside circle
	  VERTEX2F((FT_DISP_WIDTH/2)*16 - 24,(FT_DISP_HEIGHT/2)*16 - 24),
	  COLOR_RGB(0 ,0 ,0), // brown outside circle
	  VERTEX2F((FT_DISP_WIDTH/2)*16 + 24,(FT_DISP_HEIGHT/2)*16 + 24),
	  COLOR_RGB(128 ,64 ,0), // brown outside circle
	  VERTEX2F((FT_DISP_WIDTH/2)*16,(FT_DISP_HEIGHT/2)*16),
#ifdef DISPLAY_RESOLUTION_WQVGA
	  POINT_SIZE(((FT_DISP_WIDTH/4) - 19 - 50) * 16) ,
#endif
#ifdef DISPLAY_RESOLUTION_QVGA
	  POINT_SIZE(40*16) ,
#endif
	  COLOR_RGB(87 ,64 ,8), // inner circle - light brown color
	  VERTEX2F((FT_DISP_WIDTH/2)*16 - 24,(FT_DISP_HEIGHT/2)*16 - 24),
	  COLOR_RGB(0 ,0 ,0), // inner circle - light brown color
	  VERTEX2F((FT_DISP_WIDTH/2)*16 + 24,(FT_DISP_HEIGHT/2)*16+24),
	  COLOR_RGB(204 ,151 ,16), // inner circle - light brown color
	  VERTEX2F((FT_DISP_WIDTH/2)*16,(FT_DISP_HEIGHT/2)*16),
        };
        
         FT_PROGMEM prog_uint32_t ROTARY_APP_DL_BLOCK1 [ROTARY_APP_BLOCK1_SIZE_INWORDS] =   
        {     
		 CMD_DLSTART,                
		 CLEAR_COLOR_RGB(237 ,186 ,17) , // background color
		CLEAR(1,1,1),
                /* Draw two gradients, one at the top and another at the bottom. Height of former is 
                   80% of display height and later 20% of display height */
		SCISSOR_SIZE(512,218),
		SCISSOR_XY(0,0),
		//Ft_Gpu_CoCmd_Gradient(phost,0, 0, 0xEDBA11,0, FT_DispHeight, 0x82660A);
                CMD_GRADIENT,
                0,
                0xEDBA11,
                (((ft_uint32_t)FT_DISP_HEIGHT<<16)),
                0x82660A,

		SCISSOR_SIZE(512,512),
		SCISSOR_XY(0,218),
		//Ft_Gpu_CoCmd_Gradient(phost,0, 0, 0x2B2205,0, (FT_DispHeight -(FT_DispHeight*0.20))/*218*/, 0x82660A);                
                CMD_GRADIENT,
                0,
                0x2B2205,
              (((ft_uint32_t)(FT_DISP_WIDTH*0.8)<<16)),
              0x82660A,                
                };
                
                
                 FT_PROGMEM prog_uint32_t ROTARY_APP_DL_BLOCK2 [ROTARY_APP_END_FIFO_SIZE_INWORDS] =   
                {     
		  //CMD_DLSTART,  
                  STENCIL_FUNC(GEQUAL,1,255),
		  COLOR_RGB(0 ,0 ,0),
		  BEGIN(BITMAPS),
#ifdef DISPLAY_RESOLUTION_WQVGA 
	          VERTEX2II(283,182,28,'0'),VERTEX2II(248,198,28,'9'),VERTEX2II(209,195,28,'8'),VERTEX2II(176,172,28,'7'),VERTEX2II(160,137,28,'6'),
                  VERTEX2II(163,98,28,'5'),VERTEX2II(186,65,28,'4'),VERTEX2II(221,49,28,'3'),VERTEX2II(260,52,28,'2'),VERTEX2II(293,75,28,'1'),
 #endif
#ifdef DISPLAY_RESOLUTION_QVGA 
                   VERTEX2II(191,151,28,'0'),VERTEX2II(164,164,28,'9'),VERTEX2II(135,161,28,'8'),VERTEX2II(111,144,28,'7'),VERTEX2II(98,117,28,'6'),
                  VERTEX2II(101,88,28,'5'),VERTEX2II(118,64,28,'4'),VERTEX2II(145,51,28,'3'),VERTEX2II(174,54,28,'2'),VERTEX2II(198,71,28,'1'),
#endif         
		  STENCIL_OP(KEEP,KEEP),
                  STENCIL_FUNC(ALWAYS,0,255),
                  COLOR_A(128),
		
#ifdef DISPLAY_RESOLUTION_WQVGA 
	          VERTEX2II(283,182,28,'0'),VERTEX2II(248,198,28,'9'),VERTEX2II(209,195,28,'8'),VERTEX2II(176,172,28,'7'),VERTEX2II(160,137,28,'6'),
                  VERTEX2II(163,98,28,'5'),VERTEX2II(186,65,28,'4'),VERTEX2II(221,49,28,'3'),VERTEX2II(260,52,28,'2'),VERTEX2II(293,75,28,'1'),
 #endif
#ifdef DISPLAY_RESOLUTION_QVGA
                   VERTEX2II(191,151,28,'0'),VERTEX2II(164,164,28,'9'),VERTEX2II(135,161,28,'8'),VERTEX2II(111,144,28,'7'),VERTEX2II(98,117,28,'6'),
                  VERTEX2II(101,88,28,'5'),VERTEX2II(118,64,28,'4'),VERTEX2II(145,51,28,'3'),VERTEX2II(174,54,28,'2'),VERTEX2II(198,71,28,'1'),
#endif      

		  COLOR_RGB(255,255,255),
		  COLOR_A(255),
		  BEGIN(BITMAPS),		

		  /* dial image*/
		  COLOR_RGB(255,255,255),
		  TAG_MASK(1),
		  TAG(100),

		  VERTEX2II(((FT_DISP_WIDTH/2)-131),((FT_DISP_HEIGHT)-70),2,0),
		  TAG_MASK(0),	
                  /* fingerstop jpg */		
		  COLOR_RGB(255,255,255),
#ifdef DISPLAY_RESOLUTION_WQVGA
		  VERTEX2II(((FT_DISP_WIDTH/2) + 56)/*296*/,((FT_DISP_HEIGHT/2) + 17)/*153*/,0,0), //exact location of picture	
 #endif
#ifdef DISPLAY_RESOLUTION_QVGA
                   VERTEX2II(((FT_DISP_WIDTH/2) + 42)/*296*/,((FT_DISP_HEIGHT/2) + 10)/*153*/,0,0), //exact location of picture	
#endif
		  /*reject button jpg*/
		  COLOR_RGB(255,255,255),
		  TAG_MASK(1),
		  TAG(200),
		  VERTEX2II(((FT_DISP_WIDTH/2)+80),((FT_DISP_HEIGHT)-70),1,0),
		  TAG_MASK(0),
		 
                 CMD_FGCOLOR,
                 0xCC9710,

		
               CMD_BUTTON,
               ((ft_uint32_t)((ft_uint32_t)5<<16)| (FT_DISP_WIDTH/6)),               
#ifdef DISPLAY_RESOLUTION_WQVGA 
               (((ft_uint32_t)((ft_uint32_t)((FT_DISP_HEIGHT/2) - 111)<<16)|(FT_DISP_WIDTH - 160))),   
 #endif
#ifdef DISPLAY_RESOLUTION_QVGA
               ((ft_uint32_t)((ft_uint32_t)((FT_DISP_HEIGHT/2) - 95)<<16)|(FT_DISP_WIDTH - 110)),
#endif
               (((ft_uint32_t)0<<16)|28),
               0,

		
                };
        
#endif


ft_void_t Rotary_Dial_phone()
{
	ft_uint8_t ReadTag = 0,PrevTag = 0,StoreTag,flag =0, PrintTag=0,Tagcheck=0,FontWid = 0, FontHt = 0,Penup=0;
	ft_char8_t StringArray[31];	
	ft_uint32_t temp =0,tagval,PrevTh = 0,adjusting = 0, CurrTh=0;	
	ft_uint16_t OuterCircleRadius=0, InnerCircleRadius,CentreWhiteCircle,PrevTouch=0,VertexX, VertexY;
	ft_uint16_t ScreenWidthMid,ScreenHeightMid,FontPLaceX[10],FontPLaceY[10],SmallNumCircleRad, i,j,k=0,l=0,m=0,StartTheta,EndTheta,IncTheta;
	ft_uint16_t NoX,NoY,vX,MinTheta,MaxTheta,thresholdflag=0,ButHt, ButWid,FingerstopX,FingerstopY,th,theta;	
	ft_uint32_t RotaryTag=0,RamFifoStartAddr,RamFifoendsize,RamAddrOffest;
        ft_int16_t NoTouch,Touch, NextTh=0;
	SAMAPP_RotaryDial_Bmps_t BmpBalls[40];
	
	ScreenWidthMid = FT_DispWidth/2;
	ScreenHeightMid = FT_DispHeight/2; 
      	
        ft_uint32_t time = 0;

#ifdef DISPLAY_RESOLUTION_WQVGA
	OuterCircleRadius = ScreenWidthMid/2 - 19;	
	InnerCircleRadius = OuterCircleRadius - 50;
	SmallNumCircleRad = InnerCircleRadius/2 - 8;
	FingerstopX = ScreenWidthMid + 56;
	FingerstopY = ScreenHeightMid + 17;
	NoX = FT_DISP_WIDTH/6;//80; //(ScreenWidthMid)-(ScreenWidthMid/2);
	NoY = 5;//FT_DispHeight - (FT_DispHeight - 7);//7;
	ButHt = (FT_DISP_HEIGHT/2) - 111;
	ButWid = FT_DISP_WIDTH - 160;
#endif

#ifdef DISPLAY_RESOLUTION_QVGA
	OuterCircleRadius = ScreenWidthMid/2 + 5;	
	InnerCircleRadius = 30;
	SmallNumCircleRad = 13;
	FingerstopX = ScreenWidthMid + 42;
	FingerstopY = ScreenHeightMid +10;
	NoX = FT_DISP_WIDTH/6;//80; //(ScreenWidthMid)-(ScreenWidthMid/2);
	NoY = 6;//FT_DispHeight - (FT_DispHeight - 7);//7;
	ButHt = (FT_DISP_HEIGHT/2) - 95;
	ButWid = FT_DISP_WIDTH - 110;
#endif

	CentreWhiteCircle = InnerCircleRadius + ((OuterCircleRadius - InnerCircleRadius)/2);	
	VertexX = 0;
	VertexY = 0;	
	StartTheta = 50;
	EndTheta = 320;
	IncTheta = 30;
	NoTouch = -32768;
	MinTheta = 60;
	MaxTheta = 90;
	vX = ScreenWidthMid;
	StringArray[0] = '\0';
	tagval = 10;


	Ft_Gpu_Hal_WrCmd32(phost, CMD_MEMSET);
	Ft_Gpu_Hal_WrCmd32(phost, 0L);//starting address of memset
	Ft_Gpu_Hal_WrCmd32(phost, 55L);//value of memset
	Ft_Gpu_Hal_WrCmd32(phost,256*1024);
        i =0;
//        for(theta = StartTheta; theta<= EndTheta; theta+=IncTheta)
//	{		
//	    th = theta;	
//	    th = da(th);
//            temp = 0;//FontSize - 16;	
//	    if(theta  == StartTheta +30) i = 9;		
//	    j = i+'0';
//            /* To save space in arduino, the width and height of the fonts has been hardcoded */ 	   
//            FontWid = 12;/* for characters 0 to 9*/
//	    FontHt = 25;/*for all characters*/
//	    polarxy(CentreWhiteCircle,th,&VertexX,&VertexY,ScreenWidthMid,ScreenHeightMid);
//	    FontPLaceX[l] = (VertexX/16) - (FontWid/2);            
//	    FontPLaceY[l] = (VertexY/16) - (FontHt/2);            
//	    i--;
//	    l++;
//	}


	/* set the vol to max */
	Ft_Gpu_Hal_Wr8(phost, REG_VOL_SOUND,0xFF);

	/* Calculation of balls offsets and rate */
	for(i=0;i<40;i++)
	{
		BmpBalls[i].xOffset = ft_random(FT_DispWidth);
		BmpBalls[i].yOffset = ft_random(FT_DispHeight);

		BmpBalls[i].xDiff = ft_random(8) - 4;
		BmpBalls[i].yDiff = ft_random(4) - 2;
	}

	/* Construction of starting screen shot, assign all the bitmap handles here */
	Ft_App_WrCoCmd_Buffer(phost, CMD_DLSTART);
	Ft_App_WrCoCmd_Buffer(phost,CLEAR_COLOR_RGB(255,255,255));
	Ft_App_WrCoCmd_Buffer(phost,CLEAR(1,1,1));
	
	/* track circles outside the while loop */
	for(theta=StartTheta;theta<=EndTheta ;theta+=IncTheta)			
	{	
		th = theta;		
		if(theta == StartTheta + IncTheta) tagval = 9;
		th = da(th);				
		Ft_Gpu_CoCmd_Track(phost,ScreenWidthMid, ScreenHeightMid, DIAL, DIAL, tagval);		
		tagval--;
	}
	
	/* Download the raw data into intended locations */
	{
		RamAddrOffest = RAM_G;
		Ft_App_WrCoCmd_Buffer(phost, BEGIN(BITMAPS));

                RamAddrOffest = Load_RawDataFromfile(FingerStop,SAMAPP_FingerStop_SIZE,RamAddrOffest,0,ARGB4,56*2,56,38,0);//finger stopper
		RamAddrOffest = Load_RawDataFromfile(Call,SAMAPP_CallIcon,RamAddrOffest,1,ARGB4,56*2,56,56,1);//reject button
		RamAddrOffest = Load_RawDataFromfile(CallCancel,SAMAPP_CallCancelIcon,RamAddrOffest,2,ARGB4,56*2,56,56,2);//call button
		RamAddrOffest = Load_RawDataFromfile(YellowBall20x20,SAMAPP_YellowBall_20x20,RamAddrOffest,3,ARGB4,20*2,20,20,3);//ball 20x20
		RamAddrOffest = Load_RawDataFromfile(YellowBall30x30,SAMAPP_YellowBall_30x30,RamAddrOffest,4,ARGB4,30*2,30,30,4);//ball 30x30
		RamAddrOffest = Load_RawDataFromfile(YellowBall40x40,SAMAPP_YellowBall_40x40,RamAddrOffest,5,ARGB4,40*2,40,40,5);//ball 40x40
		RamAddrOffest = Load_RawDataFromfile(YellowBall60x60,SAMAPP_YellowBall_60x60,RamAddrOffest,6,ARGB4,60*2,60,60,6);//ball 60x60

		Ft_App_WrCoCmd_Buffer(phost, END());
	}


	Ft_App_WrCoCmd_Buffer(phost,DISPLAY());
	Ft_Gpu_CoCmd_Swap(phost);

	/* Download the commands into fifo */
	Ft_App_Flush_Co_Buffer(phost);

	/* Wait till coprocessor completes the operation */
	Ft_Gpu_Hal_WaitCmdfifo_empty(phost);

        /* Performance optimization - Construct static array of graphics instructions once and download into GRAM and run time use cmd_append() */
        {
          RamFifoStartAddr = RamAddrOffest;

          Ft_Gpu_Hal_WrMemFromFlash(phost,RamFifoStartAddr,(ft_prog_uchar8_t*)ROTARY_APP_DL_BLOCK0,ROTARY_APP_START_FIFO_SIZE);
             
          RamAddrOffest += ROTARY_APP_START_FIFO_SIZE;  
                
        }
        

	do{
                //Serial.println(millis()-time,DEC);   
                //time = millis();   

                /* Generate random bitmap offsets into the RGAM address directly and followed by Coprocessor append command */


                Ft_Gpu_Hal_StartTransfer(phost,FT_GPU_WRITE,RamFifoStartAddr + ROTARY_APP_START_FIFO_RANDOMBITMAP_SIZE);
		for(i = 0;i<40;i++)
		{
			Ft_Gpu_Hal_Transfer32(phost,VERTEX2II(BmpBalls[i].xOffset,BmpBalls[i].yOffset,3+(i%4),0));
			BmpBalls[i].xOffset += BmpBalls[i].xDiff;
			BmpBalls[i].yOffset += BmpBalls[i].yDiff;

			if(BmpBalls[i].xOffset < -80)
			{
				BmpBalls[i].xOffset = FT_DispWidth + 80;
			}
			else if(BmpBalls[i].xOffset > FT_DispWidth + 80)
			{
				BmpBalls[i].xOffset = -80;
			}

			if(BmpBalls[i].yOffset < -80)
			{
				BmpBalls[i].yOffset = FT_DispHeight + 80;
			}
			else if(BmpBalls[i].yOffset > FT_DispHeight + 80)
			{
				BmpBalls[i].yOffset = -80;
			}
		}
                Ft_Gpu_Hal_EndTransfer(phost);

		BmpBalls[ft_random(40)].xDiff = ft_random(8) - 4;
		BmpBalls[ft_random(40)].yDiff = ft_random(4) - 2;

                /* use coprocessor append command to download the DL commands into DL memory from graphics ram */

                /* Array optimization */





                Ft_Gpu_Hal_WrCmdBufFromFlash(phost,(ft_prog_uchar8_t*)ROTARY_APP_DL_BLOCK1,ROTARY_APP_BLOCK1_SIZE);
                Ft_Gpu_CoCmd_Append(phost,RamFifoStartAddr,ROTARY_APP_START_FIFO_SIZE);
              

		i = 0;//font inside the white circles - start at 0
		tagval = 10;// tag for the white circles - decrementing tagvalue inside loop


		Touch = Ft_Gpu_Hal_Rd16(phost, REG_TOUCH_SCREEN_XY+2);
		Tagcheck = Ft_Gpu_Hal_Rd8(phost, REG_TOUCH_TAG);

                RotaryTag = Ft_Gpu_Hal_Rd32(phost,REG_TRACKER);
		ReadTag = RotaryTag & 0xff;
                /* Anticlockwise movement */
		if((PrevTouch&0x8000) && (0 == PrevTag))
		{
			BaseTrackValInit = (ft_int16_t)((RotaryTag >> 16) & 0xffff);
		}
		else
		{
			BaseTrackValInit = 0;
		}
                /* verify for the track value */
                if((Touch != NoTouch))
		{
			if((ReadTag >= 1) && (ReadTag <= 10))
			{
				CurrTh = (ft_int16_t)((RotaryTag >> 16) & 0xffff);
                                /* BaseTrackVal is the rotary track value of the particular white circle that is moving */
				if(adjusting!=0)
				{
					flag = 0;
					BaseTrackVal += (ft_int16_t)(CurrTh - PrevTh);
					BaseTrackValSign += (ft_int16_t)(CurrTh - PrevTh);
					/* handling of loopback */
					BaseTrackVal &= 0xffff;	
                                        /* NextTh value is the track value in degrees */					
					NextTh = (ft_uint16_t)(BaseTrackValSign / 182);	
                                        NextTh = abs(NextTh);

				
				} 
				PrevTh = CurrTh;
				adjusting = (RotaryTag & 0xff);
				PrintTag = ReadTag;
			
			}
		}
                else if(Touch == NoTouch)
                {
			BaseTrackVal=0;		
			flag = 1;
                        Penup = 1;
			if(adjusting != 0)
			{        
                                /* If white circles have moved beyond their initial value, Nextth value is incremented */
				if(NextTh > 0 )
				{
                                        /* After the white circle has moved to the desired location, and when penup has been done,
                                        the white crcles moved back to its original positon of 0 degrees. */
                                        /* NextTh moves back in degrees of twenty in steps. Also sound has to be played when moving back. */
					NextTh-=20;	
					Ft_Gpu_Hal_Wr8(phost, REG_VOL_SOUND,0xff);
					Ft_Gpu_Hal_Wr16(phost, REG_SOUND,0x51);
					Ft_Gpu_Hal_Wr8(phost, REG_PLAY,1);
					MinTheta = 60;
					MaxTheta = 90;
					
				}
				else
				{ 
    					/* When the white circles have moved back to its original position, the tag value of the particular white circle 
                                               (PrintTag) is stored in the StringArray. StringArray stores upto 17 values. The thresholdflag is also checked 
                                               before entering this loop */
					NextTh = 0;
					if((PrintTag >= 1) && (PrintTag <= 10) && (thresholdflag ==1))
					{
						if (PrintTag == 10)
                                                {
                                                  PrintTag = 0;
                                                }
                                                
						StringArray[k] = PrintTag + '0';

						if (k >= 17)
						{
							k = 0;
							StringArray[k] = PrintTag + '0';
							
						}
                                                k++;
						thresholdflag=0;
					}
                            
					StringArray[k] = '\0';					
					adjusting = 0;
					
				}					
			}
                        /* When movement has been finished, the RotaryTag and Printtag are cleared*/
			else if(adjusting == 0)
			{
				RotaryTag = '\0';
				PrintTag = '\0';			
				
			}
			BaseTrackValSign = 0;
		}

                /* Sound play wrt user touch */
                /* When white circles are touched for the first time after penup - play sound */
                if((PrevTag == 0)&& ((ReadTag !=0)&&(ReadTag !=255)) && ( Penup = 1))
                {
		      Ft_Gpu_Hal_Wr8(phost, REG_VOL_SOUND,0xFF);
		      Ft_Gpu_Hal_Wr16(phost, REG_SOUND,0x51);
		      Ft_Gpu_Hal_Wr8(phost, REG_PLAY,1);
                      Penup = 0;
                }
  
		/* dial button (tag 100) & penup has been done - play dtmf sound */
		if((Tagcheck == 100)  && (flag == 1))
		{
			if(k>0)
			{				
				for(i=0;i<k;i++)
				{
					/* Play the dtmf sound */
					Ft_Gpu_Hal_Wr16(phost, REG_SOUND,StringArray[i]);
					Ft_Gpu_Hal_Wr8(phost, REG_PLAY,1);
                                        Ft_Gpu_Hal_Sleep(80);
					Ft_Gpu_Hal_Wr16(phost, REG_SOUND,0);
					Ft_Gpu_Hal_Wr8(phost, REG_PLAY,1);
					Ft_Gpu_Hal_Sleep(80);
				}
			}

		}

		/* reject button (tag -200)& penup has been done. clear the string */
		if((Tagcheck == 200) && (flag ==1) )
		{
		        StringArray[0] = '\0';
			k=0;
		}                  

                Ft_Gpu_Hal_CheckCmdBuffer(phost,ROTARY_APP_WHITECIRCELE_FIFO_SIZE);
                Ft_Gpu_Hal_StartTransfer(phost,FT_GPU_WRITE,RAM_CMD + phost->ft_cmd_fifo_wp);
		Ft_Gpu_Hal_Transfer32(phost,BEGIN(FTPOINTS));
		Ft_Gpu_Hal_Transfer32(phost,STENCIL_OP(INCR,INCR));//pass or fail increment	
		Ft_Gpu_Hal_Transfer32(phost,POINT_SIZE(SmallNumCircleRad * 16) );
		Ft_Gpu_Hal_Transfer32(phost,COLOR_A(255));
		Ft_Gpu_Hal_Transfer32(phost,COLOR_RGB(255 ,255 ,255));	

		/* Placement of number circles which will move around the rotary */		
                Ft_Gpu_Hal_Transfer32(phost,TAG_MASK(1));
		for(theta=StartTheta;theta<=EndTheta ;theta+=IncTheta)			
		{	
			
			th = theta;		
			if(theta == (StartTheta + IncTheta)) 
                        {
                          tagval = 9;	
                        }
			
			/* limit threshold level near finger stop*/
			/* for value 0 - tag is 10*/
			if((NextTh > EndTheta) && (ReadTag == 10))
			{
				NextTh = EndTheta;
				thresholdflag =1;
			}
			
			/* for value 1 to 9 - threshold*/
			for(m = 1,l=9; m<=9, l>=1;m++,l--)
			{
				if((NextTh > EndTheta - (m*IncTheta)) && (ReadTag == l))
				{
				    NextTh = EndTheta - (m*IncTheta);
		                    thresholdflag =1;
				}
			}
			/* To stop anti clockwise movement of the dial */			
			if((BaseTrackValInit + BaseTrackValSign) < BaseTrackValInit)
			{
				NextTh = 0;
			}
			th = th + NextTh;		
			
			th = da(th);	

			// calculate x and  y for placing circles according to degree
			polarxy(CentreWhiteCircle,th,&VertexX ,&VertexY ,ScreenWidthMid,ScreenHeightMid);				
		
			Ft_Gpu_Hal_Transfer32(phost,TAG(tagval));
			Ft_Gpu_Hal_Transfer32(phost,COLOR_RGB(200 ,99 ,0)); // for 3d effect adding circles of darker color
			Ft_Gpu_Hal_Transfer32(phost,VERTEX2F(VertexX-24,VertexY-24));
			Ft_Gpu_Hal_Transfer32(phost,COLOR_RGB(0 ,0 ,0)); 
			Ft_Gpu_Hal_Transfer32(phost,VERTEX2F(VertexX+24,VertexY+24));// for 3d effect adding circles of darker color
			Ft_Gpu_Hal_Transfer32(phost,COLOR_RGB(255 ,255 ,255));	
			Ft_Gpu_Hal_Transfer32(phost,VERTEX2F(VertexX,VertexY));// the base number circle

			tagval--;	
			
		}
                Ft_Gpu_Hal_Transfer32(phost,TAG_MASK(0));			
                Ft_Gpu_Hal_EndTransfer(phost);
                Ft_Gpu_Hal_Updatecmdfifo(phost,ROTARY_APP_WHITECIRCELE_FIFO_SIZE);
                
                /* When NextTh value moves between 30 degrees, play sound */
            	if((NextTh > MaxTheta) && (Touch != NoTouch))
		{
			Ft_Gpu_Hal_Wr8(phost, REG_VOL_SOUND,0xFF);
			Ft_Gpu_Hal_Wr16(phost, REG_SOUND,0x51);
			Ft_Gpu_Hal_Wr8(phost, REG_PLAY,1);
			MinTheta = MinTheta + IncTheta;
			MaxTheta = MaxTheta + IncTheta;		
		}
		else if(((NextTh < MinTheta) && (NextTh >(MinTheta-30))) && (NextTh!=0)&&(Touch != NoTouch))
		{
			Ft_Gpu_Hal_Wr8(phost, REG_VOL_SOUND,0xFF);
			Ft_Gpu_Hal_Wr16(phost, REG_SOUND,0x51);
			Ft_Gpu_Hal_Wr8(phost, REG_PLAY,1);
			MinTheta = MinTheta - IncTheta;
			MaxTheta = MaxTheta - IncTheta;		
		}
        

		/* Placement of NUmbers inside the white circle - using stencil effect*/
                RamFifoendsize = 0;
                
                Ft_Gpu_Hal_WrCmdBufFromFlash(phost,(ft_prog_uchar8_t*)ROTARY_APP_DL_BLOCK2,ROTARY_APP_END_FIFO_SIZE);
                

		/* display numbers in the centre of the point while dialing*/
		if((PrintTag >0 ) && (PrintTag <= 10) && (Touch != NoTouch))
		{
      		        Ft_Gpu_Hal_StartTransfer(phost,FT_GPU_WRITE,RAM_CMD + phost->ft_cmd_fifo_wp);
                        /* The tag value of zero font is 10. So Set PrintTag is set to zero to print it correctly */
			if (PrintTag == 10)PrintTag = 0;
			Ft_Gpu_Hal_Transfer32(phost,COLOR_RGB(0,0,0));
			
                        Ft_Gpu_Hal_Transfer32(phost,VERTEX2II((ScreenWidthMid - 16)/*225*/,(ScreenHeightMid - 27)/*110*/,31,PrintTag + '0'));

			Ft_Gpu_Hal_Transfer32(phost,COLOR_RGB(0,0,0));
			
                        Ft_Gpu_Hal_Transfer32(phost,VERTEX2II((ScreenWidthMid - 14)/*225*/,(ScreenHeightMid - 25)/*110*/,31,PrintTag + '0'));
			Ft_Gpu_Hal_Transfer32(phost,COLOR_RGB(255,255,255));
			
                        Ft_Gpu_Hal_Transfer32(phost,VERTEX2II((ScreenWidthMid - 15)/*225*/,(ScreenHeightMid - 26)/*110*/,31,PrintTag + '0'));
                        /* Restore the PrintTag to ten after printing has been done */
			if (PrintTag == 0)PrintTag = 10;
                        RamFifoendsize = 6*4;
                        Ft_Gpu_Hal_EndTransfer(phost);
        
                        Ft_Gpu_Hal_Updatecmdfifo(phost,RamFifoendsize);                    
		}

                Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(0,0,0));

		Ft_Gpu_CoCmd_Text(phost,(NoX + ButWid/2),(NoY + ButHt/2),28,OPT_CENTER,StringArray);


		Ft_App_WrCoCmd_Buffer(phost,DISPLAY());                
                
		Ft_Gpu_CoCmd_Swap(phost);

		/* Download the commands into fifo */
		Ft_App_Flush_Co_Buffer(phost);

		/* Wait till coprocessor completes the operation */
		Ft_Gpu_Hal_WaitCmdfifo_empty(phost);

                /* update all the prev with current */
		PrevTag = ReadTag;
		PrevTouch = Touch;

		}while(1);
			

}

ft_void_t Ft_BootupConfig()
{
	/* Do a power cycle for safer side */
	Ft_Gpu_Hal_Powercycle(phost,FT_TRUE);

	/* Access address 0 to wake up the FT800 */
	Ft_Gpu_HostCommand(phost,FT_GPU_ACTIVE_M);  
	Ft_Gpu_Hal_Sleep(20);

		/* Set the clk to external clock */
#ifndef ME800A_HV35R
		Ft_Gpu_HostCommand(phost,FT_GPU_EXTERNAL_OSC);
		Ft_Gpu_Hal_Sleep(10);
#endif

		{
			ft_uint8_t chipid;
			//Read Register ID to check if FT800 is ready.
			chipid = Ft_Gpu_Hal_Rd8(phost, REG_ID);
			while(chipid != 0x7C)
			{
				chipid = Ft_Gpu_Hal_Rd8(phost, REG_ID);
				printf("VC1 register ID after wake up %x\n",chipid);
				ft_delay(100);
			}
	#if defined(MSVC_PLATFORM) || defined (FT900_PLATFORM)
			printf("VC1 register ID after wake up %x\n",chipid);
	#endif
	}
	/* Configuration of LCD display */
#ifdef DISPLAY_RESOLUTION_QVGA
	/* Values specific to QVGA LCD display */
	FT_DispWidth = 320;
	FT_DispHeight = 240;
	FT_DispHCycle =  408;
	FT_DispHOffset = 70;
	FT_DispHSync0 = 0;
	FT_DispHSync1 = 10;
	FT_DispVCycle = 263;
	FT_DispVOffset = 13;
	FT_DispVSync0 = 0;
	FT_DispVSync1 = 2;
	FT_DispPCLK = 8;
	FT_DispSwizzle = 2;
	FT_DispPCLKPol = 0;
	FT_DispCSpread = 1;
	FT_DispDither = 1;

#endif
#ifdef DISPLAY_RESOLUTION_WVGA
	/* Values specific to QVGA LCD display */
	FT_DispWidth = 800;
	FT_DispHeight = 480;
	FT_DispHCycle =  928;
	FT_DispHOffset = 88;
	FT_DispHSync0 = 0;
	FT_DispHSync1 = 48;
	FT_DispVCycle = 525;
	FT_DispVOffset = 32;
	FT_DispVSync0 = 0;
	FT_DispVSync1 = 3;
	FT_DispPCLK = 2;
	FT_DispSwizzle = 0;
	FT_DispPCLKPol = 1;
	FT_DispCSpread = 0;
	FT_DispDither = 1;
#endif
#ifdef DISPLAY_RESOLUTION_HVGA_PORTRAIT
	/* Values specific to HVGA LCD display */

	FT_DispWidth = 320;
	FT_DispHeight = 480;
	FT_DispHCycle =  400;
	FT_DispHOffset = 40;
	FT_DispHSync0 = 0;
	FT_DispHSync1 = 10;
	FT_DispVCycle = 500;
	FT_DispVOffset = 10;
	FT_DispVSync0 = 0;
	FT_DispVSync1 = 5;
	FT_DispPCLK = 4;
	FT_DispSwizzle = 2;
	FT_DispPCLKPol = 1;
	FT_DispCSpread = 1;
	FT_DispDither = 1;

#endif
	Ft_Gpu_Hal_Wr16(phost, REG_HCYCLE, FT_DispHCycle);
	Ft_Gpu_Hal_Wr16(phost, REG_HOFFSET, FT_DispHOffset);
	Ft_Gpu_Hal_Wr16(phost, REG_HSYNC0, FT_DispHSync0);
	Ft_Gpu_Hal_Wr16(phost, REG_HSYNC1, FT_DispHSync1);
	Ft_Gpu_Hal_Wr16(phost, REG_VCYCLE, FT_DispVCycle);
	Ft_Gpu_Hal_Wr16(phost, REG_VOFFSET, FT_DispVOffset);
	Ft_Gpu_Hal_Wr16(phost, REG_VSYNC0, FT_DispVSync0);
	Ft_Gpu_Hal_Wr16(phost, REG_VSYNC1, FT_DispVSync1);
	Ft_Gpu_Hal_Wr8(phost, REG_SWIZZLE, FT_DispSwizzle);
	Ft_Gpu_Hal_Wr8(phost, REG_PCLK_POL, FT_DispPCLKPol);
	Ft_Gpu_Hal_Wr16(phost, REG_HSIZE, FT_DispWidth);
	Ft_Gpu_Hal_Wr16(phost, REG_VSIZE, FT_DispHeight);
	Ft_Gpu_Hal_Wr16(phost, REG_CSPREAD, FT_DispCSpread);
	Ft_Gpu_Hal_Wr16(phost, REG_DITHER, FT_DispDither);


	/* Touch configuration - configure the resistance value to 1200 - this value is specific to customer requirement and derived by experiment */
#if (defined(ME800A_HV35R) || defined(ME810A_HV35R))
    /* Touch configuration - configure the resistance value to 1200 - this value is specific to customer requirement and derived by experiment */
    Ft_Gpu_Hal_Wr16(phost, REG_TOUCH_RZTHRESH,RESISTANCE_THRESHOLD);
#endif

	Ft_Gpu_Hal_Wr8(phost, REG_GPIO_DIR,0xff);
	Ft_Gpu_Hal_Wr8(phost, REG_GPIO,0x0ff);
	Ft_Gpu_Hal_Wr8(phost, REG_GPIO,0xff);

	Ft_Gpu_Hal_WrMem(phost, RAM_DL,(ft_uint8_t *)FT_DLCODE_BOOTUP,sizeof(FT_DLCODE_BOOTUP));
	Ft_Gpu_Hal_Wr8(phost, REG_DLSWAP,DLSWAP_FRAME);
	Ft_Gpu_Hal_Wr8(phost, REG_PCLK,FT_DispPCLK);//after this display is visible on the LCD


#ifdef ENABLE_ILI9488_HVGA_PORTRAIT
	/* to cross check reset pin */
	Ft_Gpu_Hal_Wr8(phost, REG_GPIO,0xff);
	ft_delay(120);
	Ft_Gpu_Hal_Wr8(phost, REG_GPIO,0x7f);
	ft_delay(120);
	Ft_Gpu_Hal_Wr8(phost, REG_GPIO,0xff);

	ILI9488_Bootup();
#ifdef FT900_PLATFORM
	printf("aftrer ILI9488 bootup \n");
#endif

#endif

#if defined(ENABLE_ILI9488_HVGA_PORTRAIT) || defined(FT_81X_ENABLE)
	/* make the spi to quad mode - addition 2 bytes for silicon */
#ifdef FT_81X_ENABLE
	/* api to set quad and numbe of dummy bytes */
#ifdef ENABLE_SPI_QUAD
	Ft_Gpu_Hal_SetSPI(phost,FT_GPU_SPI_QUAD_CHANNEL,FT_GPU_SPI_TWODUMMY);
#elif ENABLE_SPI_DUAL
	Ft_Gpu_Hal_SetSPI(phost,FT_GPU_SPI_QUAD_CHANNEL,FT_GPU_SPI_TWODUMMY);
#else
	Ft_Gpu_Hal_SetSPI(phost,FT_GPU_SPI_SINGLE_CHANNEL,FT_GPU_SPI_ONEDUMMY);
#endif

#endif



//spi
	// Initialize SPIM HW
	sys_enable(sys_device_spi_master);
	gpio_function(27, pad_spim_sck); /* GPIO27 to SPIM_CLK */
	gpio_function(28, pad_spim_ss0); /* GPIO28 as CS */
	gpio_function(29, pad_spim_mosi); /* GPIO29 to SPIM_MOSI */
	gpio_function(30, pad_spim_miso); /* GPIO30 to SPIM_MISO */
	//gpio_dir(27, pad_dir_input);
	//gpio_dir(28, pad_dir_output);
	//gpio_dir(29, pad_dir_output);
	//gpio_dir(30, pad_dir_input);
	gpio_write(28, 1);
	spi_init(SPIM, spi_dir_master, spi_mode_0, 4);//SPISysInit(SPIM);

#ifdef ENABLE_SPI_QUAD
SPIMConfig.format         = DQSPI_TRANSFER_QUAD;			// Single data transfer

spi_option(SPIM,spi_option_bus_width,4);
printf("QUAD SPI Configured \n");
#elif ENABLE_SPI_DUAL
SPIMConfig.format         = DQSPI_TRANSFER_DUAL;			// Single data transfer
spi_option(SPIM,spi_option_bus_width,2);
printf("Dual SPI Configured \n");
#else
	//SPIMConfig.format         = DQSPI_TRANSFER_STD;			// Single data transfer
	spi_option(SPIM,spi_option_bus_width,1);
	spi_option(SPIM,spi_option_fifo_size,64);
	spi_option(SPIM,spi_option_fifo,1);
	spi_option(SPIM,spi_option_fifo_receive_trigger,1);

	printf("Single channel SPI Configured %x \n",Ft_Gpu_Hal_Rd8(phost, REG_ID));
#endif

#endif
phost->ft_cmd_fifo_wp = Ft_Gpu_Hal_Rd16(phost,REG_CMD_WRITE);
}
#ifdef FT900_PLATFORM
ft_void_t FT900_Config()
{
	sys_enable(sys_device_uart0);
		    gpio_function(48, pad_uart0_txd); /* UART0 TXD */
		    gpio_function(49, pad_uart0_rxd); /* UART0 RXD */
		    uart_open(UART0,                    /* Device */
		              1,                        /* Prescaler = 1 */
		              UART_DIVIDER_115200_BAUD,  /* Divider = 1302 */
		              uart_data_bits_8,         /* No. Data Bits */
		              uart_parity_none,         /* Parity */
		              uart_stop_bits_1);        /* No. Stop Bits */

		    /* Print out a welcome message... */
		    uart_puts(UART0,

		        "(C) Copyright 2014-2015, Future Technology Devices International Ltd. \r\n"
		        "--------------------------------------------------------------------- \r\n"
		        "Welcome to RotaryDial Example ... \r\n"
		        "\r\n"
		        "--------------------------------------------------------------------- \r\n"
		        );

		    //init_printf(UART0,myputc);

	#ifdef ENABLE_ILI9488_HVGA_PORTRAIT
		/* asign all the respective pins to gpio and set them to default values */
		gpio_function(34, pad_gpio34);
		gpio_dir(34, pad_dir_output);
	    gpio_write(34,1);

		gpio_function(27, pad_gpio27);
		gpio_dir(27, pad_dir_output);
	    gpio_write(27,1);

		gpio_function(29, pad_gpio29);
		gpio_dir(29, pad_dir_output);
	    gpio_write(29,1);

	    gpio_function(33, pad_gpio33);
	    gpio_dir(33, pad_dir_output);
	    gpio_write(33,1);


	    gpio_function(30, pad_gpio30);
	    gpio_dir(30, pad_dir_output);
	    gpio_write(30,1);

		gpio_function(28, pad_gpio28);
		gpio_dir(28, pad_dir_output);
	    gpio_write(28,1);


	  	gpio_function(43, pad_gpio43);
	  	gpio_dir(43, pad_dir_output);
	    gpio_write(43,1);
		gpio_write(34,1);
		gpio_write(28,1);
		gpio_write(43,1);
		gpio_write(33,1);
		gpio_write(33,1);

	#endif
		/* useful for timer */
		//ft_millis_init();
		//interrupt_enable_globally();
		//printf("ft900 config done \n");
	}
#endif



/* Boot up for FT800 followed by graphics primitive sample cases */
/* Initial boot up DL - make the back ground green color */

#if defined MSVC_PLATFORM | defined FT900_PLATFORM
/* Main entry point */
ft_int32_t main(ft_int32_t argc,ft_char8_t *argv[])
#endif
#if defined(ARDUINO_PLATFORM) || defined(MSVC_FT800EMU)
ft_void_t setup()
#endif
{

	ft_uint8_t chipid;
	#ifdef FT900_PLATFORM
		FT900_Config();

		sys_enable(sys_device_sd_card);
		sdhost_init();

			#define GPIO_SD_CLK  (19)
			#define GPIO_SD_CMD  (20)
			#define GPIO_SD_DAT3 (21)
			#define GPIO_SD_DAT2 (22)
			#define GPIO_SD_DAT1 (23)
			#define GPIO_SD_DAT0 (24)
			#define GPIO_SD_CD   (25)
			#define GPIO_SD_WP   (26)
		    gpio_function(GPIO_SD_CLK, pad_sd_clk); gpio_pull(GPIO_SD_CLK, pad_pull_none);//pad_pull_none
		    gpio_function(GPIO_SD_CMD, pad_sd_cmd); gpio_pull(GPIO_SD_CMD, pad_pull_pullup);
		    gpio_function(GPIO_SD_DAT3, pad_sd_data3); gpio_pull(GPIO_SD_DAT3, pad_pull_pullup);
		    gpio_function(GPIO_SD_DAT2, pad_sd_data2); gpio_pull(GPIO_SD_DAT2, pad_pull_pullup);
		    gpio_function(GPIO_SD_DAT1, pad_sd_data1); gpio_pull(GPIO_SD_DAT1, pad_pull_pullup);
		    gpio_function(GPIO_SD_DAT0, pad_sd_data0); gpio_pull(GPIO_SD_DAT0, pad_pull_pullup);
		    gpio_function(GPIO_SD_CD, pad_sd_cd); gpio_pull(GPIO_SD_CD, pad_pull_pullup);
		    gpio_function(GPIO_SD_WP, pad_sd_wp); gpio_pull(GPIO_SD_WP, pad_pull_pullup);

		    /* Check to see if a card is inserted */
	#endif
		    Ft_Gpu_HalInit_t halinit;

		    	halinit.TotalChannelNum = 1;


		    	Ft_Gpu_Hal_Init(&halinit);
		    	host.hal_config.channel_no = 0;
		    	host.hal_config.pdn_pin_no = FT800_PD_N;
		    	host.hal_config.spi_cs_pin_no = FT800_SEL_PIN;
		    #ifdef MSVC_PLATFORM_SPI
		    	host.hal_config.spi_clockrate_khz = 12000; //in KHz
		    #endif
		    #ifdef ARDUINO_PLATFORM_SPI
		    	host.hal_config.spi_clockrate_khz = 4000; //in KHz
		    #endif
		        Ft_Gpu_Hal_Open(&host);


		    	phost = &host;

		        Ft_BootupConfig();

	#ifdef FT900_PLATFORM

	    /* Change clock freuency to 25mhz */
		spi_init(SPIM, spi_dir_master, spi_mode_0, 4);
		spi_option(SPIM,spi_option_bus_width,1);
		spi_option(SPIM,spi_option_fifo_size,64);
		spi_option(SPIM,spi_option_fifo,1);
		spi_option(SPIM,spi_option_fifo_receive_trigger,1);
	#endif
	
	#if ((defined FT900_PLATFORM) || defined(MSVC_PLATFORM))
		printf("\n reg_touch_rz =0x%x ", Ft_Gpu_Hal_Rd16(phost, REG_TOUCH_RZ));
		printf("\n reg_touch_rzthresh =0x%x ", Ft_Gpu_Hal_Rd32(phost, REG_TOUCH_RZTHRESH));
	    printf("\n reg_touch_tag_xy=0x%x",Ft_Gpu_Hal_Rd32(phost, REG_TOUCH_TAG_XY));
		printf("\n reg_touch_tag=0x%x",Ft_Gpu_Hal_Rd32(phost, REG_TOUCH_TAG));
	#endif

#ifdef FT900_PLATFORM
	    SDHOST_STATUS sd_status;
	  //

		if (sdhost_card_detect() != SDHOST_CARD_INSERTED)
		{
		printf("Please Insert SD Card\r\n");
		}
		else
		{ printf("\n SD Card inserted!");}


		if (f_mount(&FatFs, "", 0) != FR_OK)
		{
			printf("\n Mounted failed.");;
		}

			printf("\n Mounted succesfully.");
#endif

    /*It is optional to clear the screen here*/	
    Ft_Gpu_Hal_WrMem(phost, RAM_DL,(ft_uint8_t *)FT_DLCODE_BOOTUP,sizeof(FT_DLCODE_BOOTUP));
    Ft_Gpu_Hal_Wr8(phost, REG_DLSWAP,DLSWAP_FRAME);
    
    Ft_Gpu_Hal_Sleep(1000);//Show the booting up screen. );
	printf("reg_touch_tag=0x%x",Ft_Gpu_Hal_Rd32(phost, REG_TOUCH_TAG));

	Info();
	
	Rotary_Dial_phone();
	  

#if defined(MSVC_PLATFORM) || defined(MSVC_FT800EMU)
	/* Audio playback api*/    
	Ft_Gpu_Hal_Wr8(phost, REG_VOL_SOUND,255);

#endif




	/* Close all the opened handles */
    Ft_Gpu_Hal_Close(phost);
    Ft_Gpu_Hal_DeInit();
#ifdef MSVC_PLATFORM
	return 0;
#endif
}

void loop()
{
}



/* Nothing beyond this */















